import Config from "./Config";
import FastifyCookie from "@fastify/cookie";
import FunkyArray from "funky-array";
import SessionUser from "./SessionUser";
import { FastifyReply, FastifyRequest } from "fastify";
import User from "../entities/User";
import { randomBytes } from "crypto";

type Cookies = { [cookieName: string]: string | undefined }

export default abstract class Session {
	public static Sessions = new FunkyArray<string, SessionUser>();
	public static SessionExpiryInterval = setInterval(() => {
		const currentTime = Date.now();
		for (const key of Session.Sessions.keys) {
			const session = Session.Sessions.get(key);
			if (!session || (session && currentTime >= session.validityPeriod.getTime())) {
				Session.Sessions.remove(key);
			}
		}
	}, 3600000);

	public static AssignUserSession(res:FastifyReply, user:User) {
		const validPeriod = new Date();
		validPeriod.setTime(validPeriod.getTime() + Config.session.validity);
		const key = randomBytes(Config.session.length).toString("hex");

		Session.Sessions.set(key, new SessionUser(user.Id, user.Username, user.UserType, validPeriod));

		res.setCookie("EHP_SESSION", key, {
			path: "/",
			signed: true
		});
	}

	public static Clear(cookies:Cookies, res:FastifyReply) {
		if ("EHP_SESSION" in cookies && typeof(cookies["EHP_SESSION"]) === "string") {
			const key:unknown = FastifyCookie.unsign(cookies["EHP_SESSION"], Config.session.secret);
			Session.Sessions.remove(key as string);

			res.clearCookie("EHP_SESSION");
		}
	}

	public static CheckValiditiy(cookies:Cookies) {
		if ("EHP_SESSION" in cookies && typeof(cookies["EHP_SESSION"]) === "string") {
			const key = FastifyCookie.unsign(cookies["EHP_SESSION"], Config.session.secret);
			if (key.valid && Session.Sessions.has(key.value ?? "badkey")) {
				return Session.Sessions.get(key.value ?? "badkey");
			}
		}
	
		return undefined;
	}
}